class generator_spy:
    """
    >>> @generator_spy
    ... def strange():
    ...     try:
    ...         yield 'good'
    ...     except Exception as e:
    ...         yield 'bad'
    ...     yield 'done'
    >>> gen = strange()
    creating child…
    …done
    >>> next(gen)
    caller used next()
    generator yielded good
    'good'
    >>> gen.throw(ValueError)
    caller threw ValueError
    generator yielded bad
    'bad'
    >>> next(gen)
    caller used next()
    generator yielded done
    'done'
    >>> gen.close()
    caller killed the generator
    generator dead
    """

    def __init__(self, generator):
        self.generator = generator

    def __call__(self):
        print('creating child…')
        self.child = self.generator()
        print('…done')
        return self

    def __iter__(self):
        return self

    def _send(self, op, *args): # unused
        try:
            ans = op(*args)
        except Exception as e:
            print('generator raised', e)
            raise
        else:
            print('generator yielded', ans)
        return ans

    def __next__(self):
        print('caller used next()')
        return self._send(next, self.child)

    def send(self, what):
        print('caller sent', what)
        return self._send(self.child.send, what)

    def throw(self, type, value=None, traceback=None):
        print('caller threw', type.__name__)
        return self._send(self.child.throw, type, value, traceback)

    def close(self):
        print('caller killed the generator')
        self.child.close()
        print('generator dead')
