def deprecated__f(func):
    """Print a warning about deprection once

    Installs a wrapper function which prints a warning on first
    function use.
    """
    count = 0
    def wrapper(*args, **kwargs):
        nonlocal count
        count += 1
        if count == 1:
            print(func.__name__, 'is deprecated')
        return func(*args, **kwargs)
    return wrapper

class deprecated:
    """Print a deprecation warning once

    Installs a wrapper function which
    prints a warning on first function use.

    >>> @deprecated()
    ... def f(): return 3
    >>> f()
    f is deprecated
    3
    >>> f()
    3
    """
    def __init__(self):
        pass
    def __call__(self, func):
        self.func = func
        self.count = 0
        return self.wrapper
    def wrapper(self, *args, **kwargs):
        self.count += 1
        if self.count == 1:
            print(self.func.__name__, 'is deprecated')
        return self.func(*args, **kwargs)


def deprecated2_(message):
    """
    >>> @deprecated2_('function {func.__name__} is deprecated')
    ... def eot(): return 'EOT'
    >>> eot()
    function eot is deprecated
    'EOT'
    """
    def _deprecated(func):
        count = 0
        def wrapper(*args, **kwargs):
            nonlocal count
            count += 1
            if count == 1:
                print(message.format(func=func))
            return func(*args, **kwargs)
        return wrapper
    return _deprecated

class deprecated2:
    """
    >>> @deprecated2('function {func.__name__} is deprecated')
    ... def eot(): return 'EOT'
    >>> eot()
    function eot is deprecated
    'EOT'
    """
    def __init__(self, message):
        self.message = message
    def __call__(self, func):
        self.func = func
        self.count = 0
        return self.wrapper
    def wrapper(self, *args, **kwargs):
        self.count += 1
        if self.count == 1:
            print(self.message.format(func=self.func))
        return self.func(*args, **kwargs)

def deprecated3(message, substitute):
    def _deprecated(func):
        count = 0
        def wrapper(*args, **kwargs):
            nonlocal count
            count += 1
            if count == 1:
                print(message.format(func=func))
            return substitute(*args, **kwargs)
        return wrapper
    return _deprecated
