def deprecated3(message, other_func=None):
    """print a message about deprecation once and call the original function

    >>> def eot_new(): return 'EOT NEW'
    >>> @deprecated3('using eot_new not {func.__name__}', eot_new)
    ... def eot(): return 'EOT'
    >>> eot()
    using eot_new not eot
    'EOT NEW'
    """
    def _deprecated(func):
        count = 0
        repl_func = other_func if other_func else func
        def wrapper(*args, **kwargs):
            nonlocal count
            count += 1
            if count == 1:
                print(message.format(func=func))
            return repl_func(*args, **kwargs)
        return wrapper
    return _deprecated
