import functools

def logged(func):
    """Log arguments before function call and afterwards the returned value

    >>> @logged
    ... def func(*args):
    ...     return 3 + len(args)
    >>> func(4, 4, 4)
    you called func(4, 4, 4)
    it returned 6
    6

    >>> @logged
    ... def func2(a=None, b=None):
    ...     return None
    >>> func2()
    you called func2()
    it returned None
    >>> func2(3, b=2)
    you called func2(3, b=2)
    it returned None

    >>> @logged
    ... def func3():
    ...     "this function is documented"
    ...     pass
    >>> print(func3.__doc__)
    this function is documented
    """

    def wrapper(*args, **kwargs):
        print('you called {.__name__}({}{}{})'.format(
                func,
                str(list(args))[1:-1], # cast to list is because one-tuple has an extra ,
                ', ' if kwargs else '',
                ', '.join('{}={}'.format(*pair) for pair in kwargs.items()),
                ))
        val = func(*args, **kwargs)
        print('it returned', val)
        return val
    return functools.update_wrapper(wrapper, func)

