#######################################################################
# This script compares the speed of the computation of a polynomial
# for different (numpy and numexpr) in-memory paradigms.
#
# Author: Francesc Alted
# Date: 2010-10-01
#######################################################################

from time import time
import numpy as np
import numexpr as ne


expr = ".25*x**3 + .75*x**2 - 1.5*x - 2"  # the polynomial to compute
#expr = "((.25*x + .75)*x - 1.5)*x - 2"   # a computer-friendly polynomial
#expr = "sin(x)**2+cos(x)**2"             # a transcendental function
N = 10*1000*1000            # the number of points to compute expression
x = np.linspace(-1, 1, N)   # the x in range [-1, 1]

what = "numpy"              # uses numpy for computations
#what = "numexpr"           # uses numexpr for computations

ne.set_num_threads(1)       # the number of threads for numexpr computations

def compute():
    """Compute the polynomial."""
    global expr
    if what == "numpy":
        if "sin" in expr:
            # Trick to allow numpy evaluate this
            expr = "np.sin(x)**2+np.cos(x)**2"
        y = eval(expr)
    else:
        y = ne.evaluate(expr)
    return len(y)


if __name__ == '__main__':
    print "Computing: '%s' using %s with %d points" % (expr, what, N)
    t0 = time()
    result = compute()
    ts = round(time() - t0, 3)
    print "*** Time elapsed:", ts
